function model_durations(basename, eventsperfile, Nfluor, ratediff)

% function model_durations(basename, eventsperfile, Nfluor, ratediff)
%
% This program creates modeled fluorescence correlation spectroscopy
% (FCS) data.  The modeled data consists of a vector of durations (a 
% duration is defined as the length of time between consecutive detected 
% events).  The modeled data is saved in a format consistent with 
% data collected during the Chem184 FCS lab.
% 
% We assume that, once a fluorophore enters the confocal volume (CV), 
% the time it spends within the CV is described by a Gaussian with mean 
% 0.1 ms and standard deviation 0.05 ms.  The sources of detection events 
% are (1) emission of a photon by a fluor in the CV and (2) dark/background 
% noise. Because both of these are Poisson processes, the distribution of their 
% durations is exponential.  The time constant of the exponential decay for the 
% former is set to 1 us and for the latter to 1 ms.
%
% INPUTS:
%     basename - base name of your data files (string); the name of your files 
%            will be basename1, basename2, basename3, ... Data is saved in a folder
%            with the name: "basename" in the current directory
%     Nfluor - the number of fluorophores that will pass through your 
%            confocal volume during during the modeled dataset 
%     eventsperfile - the number of durations saved in each
%            data file; the total number of  files saved will be the total 
%            number of modeled durations divided by eventsperfile.
%     ratediff - the rate at which a fluor diffuses into the confocal volume (s^-1).  
%            Altering this is equivalent to altering the concentration of 
%            your fluor sample  
% 
%                                               DAVID ALTMAN 6/18/07

confocaltime = 10^(-4);     % how long a fluor spends in the confocal volume (s)
stdconfocal = 0.5*10^(-4);  % standard devaition of the length of time in the confocal volume (s)
temit = 10^(-6);            % the mean time between photon emissions for a fluor in the confocal volume  (s)
tdiff = 1/ratediff;         % the mean time between fluors diffusing into the confocal volume (s)
tdark = 10^(-3);            % the mean time between dark count/background detections (s)

% create a list of times when a fluor enters the confocal volume.  
fluors = exprnd(tdiff, [Nfluor, 1]);
fluors = cumsum(fluors);

% Create a list of times when a fluor in the confocal volume emits a photon.
time = 0; 
eventtimes = [];
for i = 1:Nfluor
    begintime = fluors(i);                                % time when the fluor enters the confocal volume
    conftime = normrnd(confocaltime, stdconfocal, [1, 1]);    % determine how long the fluor spends in the confocal volume
    interval = 0;                                         
    flag = 0;
    while flag == 0         
        emitduration = exprnd(temit, [1, 1]);       % measure the next duration arising from photon detection 
        interval = interval + emitduration;         % how much time has passed since entering the confocal volume
        time = time + emitduration;                    
        if interval > conftime                      % ASK: is the fluor still in the confocal volume?    
            flag = 1;                               %    if NO, then stop collecting duration measurements
        else
            eventtimes = [eventtimes, time];        %    if YES, then add this event to our list of measurements
        end
    end    
end
    
% Add to our list of detection times those events arising from the dark/backgorund noise
totaltime = eventtimes(end);                    % total length of data collection
time = 0;                                 
flag = 0;
while flag == 0
   darkcount = exprnd(tdark, [1, 1]);           % measure the next duration arising from dark noise
   time = time + darkcount;  
   if time > totaltime                          % ASK: are we at the end of the data collection?
        flag = 1;                               %    if NO, then stop collecting noise measurements
   else
        eventtimes = [eventtimes, time];        %    if YES, then add this event to our list of measurements
   end
end

Nsignals = length(eventtimes);                 % total number of detection events (photon+noise)
plot(eventtimes, ones(1, Nsignals), '*b')       
  
durations = sort(eventtimes);                  % sort the list of detection events by time
durations = diff(durations);                   % calculate the durations that describe our data set
Ndurations = length(durations);                % total number of duartion events (Nsignals-1)

% Parse the durations and save the data files in a folder with the name:
% "basename"
disp(['Number of files to be saved: ', num2str(floor(Ndurations/eventsperfile)-1)]); 
mkdir(basename);
cd(basename);
filedur = [];              % this will be a list of mean durations for each file 
filelength = [];           % this will be a list of total length of time for each file

% The durations are divided into lists of length "eventsperfile" 
for i = 1:floor(Ndurations/eventsperfile)-1
    filename = [basename, num2str(i-1)];
    eventsubset = round(durations((i*eventsperfile):((i+1)*eventsperfile-1) )*10^9);    % the data is converted into integers
                                                                            % and units are in ns        
    fid = fopen(filename,'w');
    fprintf(fid,'%f\n', eventsubset);
    fclose(fid);
    filedur = [filedur, mean(eventsubset)];
    filelength = [filelength, sum(eventsubset)];
end

% Display information about the modeled datatset - using ms instead of ns
% time-scale
disp(['Mean duration: ', num2str(mean(filedur)/10^6), ' ms    Number of durations: ',num2str(eventsperfile) ]);
disp(['Mean filelength: ', num2str(mean(filelength)/10^6), ' ms']);
disp('  ')
disp(['How often a fluor passes into the confocal volume: ', num2str((tdiff)*10^(3)), ' ms     N events: ', num2str(Nfluor)]);
disp(['How often a fluor emits a photon: ', num2str((temit)*10^(3)), ' ms']);
disp(['How often a background signal occurs: ', num2str((tdark)*10^3), ' ms'])
disp(['How long a fluor stays in the confocal volume: ', num2str((confocaltime)*10^(3)), ' +/- ', num2str(stdconfocal*10^3), ' ms']);
    
